@@ -1,5 +1,6 @@
 /****************************************************************************
   Copyright (C) 2012 Monty Program AB
+                2013, 2022 MariaDB Corporation AB
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -56,10 +57,11 @@
 #include <mysql/client_plugin.h>
 #include <ma_common.h>
 #include "ma_priv.h"
+#include <assert.h>
 
 
 #define UPDATE_STMT_ERROR(stmt)\
-stmt_set_error((stmt), (stmt)->mysql->net.last_errno, (stmt)->mysql->net.sqlstate, (stmt)->mysql->net.last_error)
+SET_CLIENT_STMT_ERROR((stmt), (stmt)->mysql->net.last_errno, (stmt)->mysql->net.sqlstate, (stmt)->mysql->net.last_error)
 
 #define STMT_NUM_OFS(type, a, r) (((type *)(a))[r])
 #define MADB_RESET_ERROR     1
@@ -89,34 +91,18 @@ void stmt_set_error(MYSQL_STMT *stmt,
                   ...)
 {
   va_list ap;
+  const char *error= NULL;
 
-  const char *errmsg= format;
+  if (error_nr >= CR_MIN_ERROR && error_nr <= CR_MYSQL_LAST_ERROR)
+    error= ER(error_nr);
+  else if (error_nr >= CER_MIN_ERROR && error_nr <= CR_MARIADB_LAST_ERROR)
+    error= CER(error_nr);
 
   stmt->last_errno= error_nr;
   ma_strmake(stmt->sqlstate, sqlstate, SQLSTATE_LENGTH);
-
-  if (!format)
-  {
-    if (IS_MYSQL_ERROR(error_nr) || IS_MARIADB_ERROR(error_nr))
-      errmsg= ER(error_nr);
-    else {
-      snprintf(stmt->last_error, MYSQL_ERRMSG_SIZE - 1,
-               ER_UNKNOWN_ERROR_CODE, error_nr);
-      return;
-    }
-  }
-
-  /* Fix for CONC-627: If this is a server error message, we don't
-     need to substitute and possible variadic arguments will be
-     ignored */
-  if (!IS_MYSQL_ERROR(error_nr) && !IS_MARIADB_ERROR(error_nr))
-  {
-    strncpy(stmt->last_error, format, MYSQL_ERRMSG_SIZE - 1);
-    return;
-  }
-
   va_start(ap, format);
-  vsnprintf(stmt->last_error, MYSQL_ERRMSG_SIZE - 1, errmsg, ap);
+  vsnprintf(stmt->last_error, MYSQL_ERRMSG_SIZE,
+            format ? format : error ? error : "", ap);
   va_end(ap);
   return;
 }
@@ -221,7 +207,7 @@ int mthd_stmt_read_all_rows(MYSQL_STMT *stmt)
       /* allocate space for rows */
       if (!(current= (MYSQL_ROWS *)ma_alloc_root(&result->alloc, sizeof(MYSQL_ROWS) + packet_len)))
       {
-        stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+        SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
         return(1);
       }
       current->data= (MYSQL_ROW)(current + 1);
@@ -300,18 +286,20 @@ int mthd_stmt_read_all_rows(MYSQL_STMT *stmt)
       result->rows++;
     } else  /* end of stream */
     {
+      unsigned int last_status= stmt->mysql->server_status;
       *pprevious= 0;
       /* sace status info */
       p++;
       stmt->upsert_status.warning_count= stmt->mysql->warning_count= uint2korr(p);
       p+=2;
       stmt->upsert_status.server_status= stmt->mysql->server_status= uint2korr(p);
+      ma_status_callback(stmt->mysql, last_status);
       stmt->result_cursor= result->data;
       return(0);
     }
   }
   stmt->result_cursor= 0;
-  stmt_set_error(stmt, stmt->mysql->net.last_errno, stmt->mysql->net.sqlstate,
+  SET_CLIENT_STMT_ERROR(stmt, stmt->mysql->net.last_errno, stmt->mysql->net.sqlstate,
       stmt->mysql->net.last_error);
   return(1);
 }
@@ -323,7 +311,7 @@ static int stmt_cursor_fetch(MYSQL_STMT *stmt, uchar **row)
 
   if (stmt->state < MYSQL_STMT_USE_OR_STORE_CALLED)
   {
-    stmt_set_error(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -348,10 +336,13 @@ static int stmt_cursor_fetch(MYSQL_STMT *stmt, uchar **row)
     result->data= 0;
     result->rows= 0;
 
-    if (stmt->mysql->methods->db_stmt_read_all_rows(stmt))
-      return(1);
+    if (!stmt->mysql->options.extension->skip_read_response)
+    {
+      if (stmt->mysql->methods->db_stmt_read_all_rows(stmt))
+        return(1);
 
-    return(stmt_buffered_fetch(stmt, row));
+      return(stmt_buffered_fetch(stmt, row));
+    }
   }
   /* no more cursor data available */
   *row= NULL;
@@ -366,13 +357,16 @@ void mthd_stmt_flush_unbuffered(MYSQL_STMT *stmt)
                     stmt->state < MYSQL_STMT_FETCH_DONE;
   while ((packet_len = ma_net_safe_read(stmt->mysql)) != packet_error)
   {
+    unsigned int last_status= stmt->mysql->server_status;
     uchar *pos= stmt->mysql->net.read_pos;
+
     if (!in_resultset && *pos == 0) /* OK */
     {
       pos++;
       net_field_length(&pos);
       net_field_length(&pos);
       stmt->mysql->server_status= uint2korr(pos);
+      ma_status_callback(stmt->mysql, last_status);
       goto end;
     }
     if (packet_len < 8 && *pos == 254) /* EOF */
@@ -380,6 +374,7 @@ void mthd_stmt_flush_unbuffered(MYSQL_STMT *stmt)
       if (mariadb_connection(stmt->mysql))
       {
         stmt->mysql->server_status= uint2korr(pos + 3);
+        ma_status_callback(stmt->mysql, last_status);
         if (in_resultset)
           goto end;
         in_resultset= 1;
@@ -589,7 +584,7 @@ int store_param(MYSQL_STMT *stmt, int column, unsigned char **p, unsigned long r
        1          1       negative
        2-5        4       day
        6          1       hour
-       7          1       ninute
+       7          1       minute
        8          1       second;
        9-13       4       second_part
        */
@@ -690,14 +685,14 @@ int store_param(MYSQL_STMT *stmt, int column, unsigned char **p, unsigned long r
 
   default:
     /* unsupported parameter type */
-    stmt_set_error(stmt, CR_UNSUPPORTED_PARAM_TYPE, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_UNSUPPORTED_PARAM_TYPE, SQLSTATE_UNKNOWN, 0);
     return 1;
   }
   return 0;
 }
 
-/* {{{ mysqlnd_stmt_execute_generate_simple_request */
-unsigned char* mysql_stmt_execute_generate_simple_request(MYSQL_STMT *stmt, size_t *request_len)
+/* {{{ ma_stmt_execute_generate_simple_request */
+unsigned char* ma_stmt_execute_generate_simple_request(MYSQL_STMT *stmt, size_t *request_len)
 {
   /* execute packet has the following format:
      Offset   Length      Description
@@ -723,7 +718,6 @@ unsigned char* mysql_stmt_execute_generate_simple_request(MYSQL_STMT *stmt, size
   size_t length= 1024;
   size_t free_bytes= 0;
   size_t null_byte_offset= 0;
-  uchar *tmp_start;
   uint i;
 
   uchar *start= NULL, *p;
@@ -752,9 +746,8 @@ unsigned char* mysql_stmt_execute_generate_simple_request(MYSQL_STMT *stmt, size
     {
       size_t offset= p - start;
       length+= offset + null_count + 20;
-      if (!(tmp_start= (uchar *)realloc(start, length)))
+      if (!(start= (uchar *)realloc(start, length)))
         goto mem_error;
-      start= tmp_start;
       p= start + offset;
     }
 
@@ -776,9 +769,8 @@ unsigned char* mysql_stmt_execute_generate_simple_request(MYSQL_STMT *stmt, size
       {
         size_t offset= p - start;
         length= offset + stmt->param_count * 2 + 20;
-        if (!(tmp_start= (uchar *)realloc(start, length)))
+        if (!(start= (uchar *)realloc(start, length)))
           goto mem_error;
-        start= tmp_start;
         p= start + offset;
       }
       for (i = 0; i < stmt->param_count; i++)
@@ -847,9 +839,8 @@ unsigned char* mysql_stmt_execute_generate_simple_request(MYSQL_STMT *stmt, size
       {
         size_t offset= p - start;
         length= MAX(2 * length, offset + size + 20);
-        if (!(tmp_start= (uchar *)realloc(start, length)))
+        if (!(start= (uchar *)realloc(start, length)))
           goto mem_error;
-        start= tmp_start;
         p= start + offset;
       }
       if (((stmt->params[i].is_null && *stmt->params[i].is_null) ||
@@ -870,7 +861,7 @@ unsigned char* mysql_stmt_execute_generate_simple_request(MYSQL_STMT *stmt, size
   *request_len = (size_t)(p - start);
   return start;
 mem_error:
-  stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+  SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
   free(start);
   *request_len= 0;
   return NULL;
@@ -891,8 +882,8 @@ my_bool mysql_stmt_skip_paramset(MYSQL_STMT *stmt, uint row)
 }
 /* }}} */
 
-/* {{{ mysql_stmt_execute_generate_bulk_request */
-unsigned char* mysql_stmt_execute_generate_bulk_request(MYSQL_STMT *stmt, size_t *request_len)
+/* {{{ ma_stmt_execute_generate_bulk_request */
+unsigned char* ma_stmt_execute_generate_bulk_request(MYSQL_STMT *stmt, size_t *request_len)
 {
   /* execute packet has the following format:
      Offset   Length      Description
@@ -922,7 +913,6 @@ unsigned char* mysql_stmt_execute_generate_bulk_request(MYSQL_STMT *stmt, size_t
   size_t length= 1024;
   size_t free_bytes= 0;
   ushort flags= 0;
-  uchar *tmp_start;
   uint i, j;
 
   uchar *start= NULL, *p;
@@ -955,7 +945,7 @@ unsigned char* mysql_stmt_execute_generate_bulk_request(MYSQL_STMT *stmt, size_t
   p+=2;
 
   /* When using mariadb_stmt_execute_direct stmt->paran_count is
-     not knowm, so we need to assign prebind_params, which was previously
+     not known, so we need to assign prebind_params, which was previously
      set by mysql_stmt_attr_set
   */
   if (!stmt->param_count && stmt->prebind_params)
@@ -974,9 +964,8 @@ unsigned char* mysql_stmt_execute_generate_bulk_request(MYSQL_STMT *stmt, size_t
       {
         size_t offset= p - start;
         length= offset + stmt->param_count * 2 + 20;
-        if (!(tmp_start= (uchar *)realloc(start, length)))
+        if (!(start= (uchar *)realloc(start, length)))
           goto mem_error;
-        start= tmp_start;
         p= start + offset;
       }
       for (i = 0; i < stmt->param_count; i++)
@@ -1061,9 +1050,8 @@ unsigned char* mysql_stmt_execute_generate_bulk_request(MYSQL_STMT *stmt, size_t
         {
           size_t offset= p - start;
           length= MAX(2 * length, offset + size + 20);
-          if (!(tmp_start= (uchar *)realloc(start, length)))
+          if (!(start= (uchar *)realloc(start, length)))
             goto mem_error;
-          start= tmp_start;
           p= start + offset;
         }
 
@@ -1080,12 +1068,46 @@ unsigned char* mysql_stmt_execute_generate_bulk_request(MYSQL_STMT *stmt, size_t
   *request_len = (size_t)(p - start);
   return start;
 mem_error:
-  stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+  SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
   free(start);
   *request_len= 0;
   return NULL;
 }
 /* }}} */
+
+
+unsigned char* ma_stmt_execute_generate_request(MYSQL_STMT *stmt, size_t *request_len, my_bool internal)
+{
+  unsigned char *buf;
+
+
+  if (stmt->request_buffer)
+  {
+    *request_len= stmt->request_length;
+    buf= stmt->request_buffer;
+    /* store actual stmt id */
+    int4store(buf, stmt->stmt_id);
+    /* clear buffer, memory will be freed in execute */
+    stmt->request_buffer= NULL;
+    stmt->request_length= 0;
+    return buf;
+  }
+  if (stmt->array_size > 0)
+    buf= ma_stmt_execute_generate_bulk_request(stmt, request_len);
+  else
+    buf= ma_stmt_execute_generate_simple_request(stmt, request_len);
+
+  if (internal)
+  {
+    if (stmt->request_buffer)
+      free(stmt->request_buffer);
+    stmt->request_buffer= buf;
+    stmt->request_length= *request_len;
+  }
+  return buf;
+}
+
+
 /*!
  *******************************************************************************
 
@@ -1143,7 +1165,7 @@ my_bool STDCALL mysql_stmt_attr_set(MYSQL_STMT *stmt, enum enum_stmt_attr_type a
   case STMT_ATTR_CURSOR_TYPE:
     if (*(ulong *)value > (unsigned long) CURSOR_TYPE_READ_ONLY)
     {
-      stmt_set_error(stmt, CR_NOT_IMPLEMENTED, SQLSTATE_UNKNOWN, 0);
+      SET_CLIENT_STMT_ERROR(stmt, CR_NOT_IMPLEMENTED, SQLSTATE_UNKNOWN, 0);
       return(1);
     }
     stmt->flags = *(ulong *)value;
@@ -1162,7 +1184,7 @@ my_bool STDCALL mysql_stmt_attr_set(MYSQL_STMT *stmt, enum enum_stmt_attr_type a
       stmt->state= MYSQL_STMT_INITTED;
       stmt->params= 0;
     }
-    stmt->prebind_params= *(unsigned int *)value;
+    stmt->prebind_params= stmt->param_count= *(unsigned int *)value;
     break;
   case STMT_ATTR_ARRAY_SIZE:
     stmt->array_size= *(unsigned int *)value;
@@ -1180,7 +1202,7 @@ my_bool STDCALL mysql_stmt_attr_set(MYSQL_STMT *stmt, enum enum_stmt_attr_type a
     stmt->user_data= (void *)value;
     break;
   default:
-    stmt_set_error(stmt, CR_NOT_IMPLEMENTED, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_NOT_IMPLEMENTED, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
   return(0);
@@ -1192,7 +1214,7 @@ my_bool STDCALL mysql_stmt_bind_param(MYSQL_STMT *stmt, MYSQL_BIND *bind)
 
   if (!mysql)
   {
-    stmt_set_error(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -1206,7 +1228,7 @@ my_bool STDCALL mysql_stmt_bind_param(MYSQL_STMT *stmt, MYSQL_BIND *bind)
     {
       if (!(stmt->params= (MYSQL_BIND *)ma_alloc_root(&stmt->mem_root, stmt->prebind_params * sizeof(MYSQL_BIND))))
       {
-        stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+        SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
         return(1);
       }
       memset(stmt->params, '\0', stmt->prebind_params * sizeof(MYSQL_BIND));
@@ -1214,7 +1236,7 @@ my_bool STDCALL mysql_stmt_bind_param(MYSQL_STMT *stmt, MYSQL_BIND *bind)
     stmt->param_count= stmt->prebind_params;
   }
   else if (stmt->state < MYSQL_STMT_PREPARED) {
-    stmt_set_error(stmt, CR_NO_PREPARE_STMT, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_NO_PREPARE_STMT, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -1230,7 +1252,7 @@ my_bool STDCALL mysql_stmt_bind_param(MYSQL_STMT *stmt, MYSQL_BIND *bind)
       if (stmt->mysql->methods->db_supported_buffer_type &&
           !stmt->mysql->methods->db_supported_buffer_type(stmt->params[i].buffer_type))
       {
-        stmt_set_error(stmt, CR_UNSUPPORTED_PARAM_TYPE, SQLSTATE_UNKNOWN, 0);
+        SET_CLIENT_STMT_ERROR(stmt, CR_UNSUPPORTED_PARAM_TYPE, SQLSTATE_UNKNOWN, 0);
         return(1);
       }
       if (!stmt->params[i].is_null)
@@ -1282,7 +1304,7 @@ my_bool STDCALL mysql_stmt_bind_param(MYSQL_STMT *stmt, MYSQL_BIND *bind)
       case MYSQL_TYPE_NEWDECIMAL:
         break;
       default:
-        stmt_set_error(stmt, CR_UNSUPPORTED_PARAM_TYPE, SQLSTATE_UNKNOWN, 0);
+        SET_CLIENT_STMT_ERROR(stmt, CR_UNSUPPORTED_PARAM_TYPE, SQLSTATE_UNKNOWN, 0);
         return(1);
         break;
       }
@@ -1300,13 +1322,13 @@ my_bool STDCALL mysql_stmt_bind_result(MYSQL_STMT *stmt, MYSQL_BIND *bind)
 
   if (stmt->state < MYSQL_STMT_PREPARED)
   {
-    stmt_set_error(stmt, CR_NO_PREPARE_STMT, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_NO_PREPARE_STMT, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
   if (!stmt->field_count)
   {
-    stmt_set_error(stmt, CR_NO_STMT_METADATA, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_NO_STMT_METADATA, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -1323,7 +1345,7 @@ my_bool STDCALL mysql_stmt_bind_result(MYSQL_STMT *stmt, MYSQL_BIND *bind)
                 &((MADB_STMT_EXTENSION *)stmt->extension)->fields_ma_alloc_root;
     if (!(stmt->bind= (MYSQL_BIND *)ma_alloc_root(fields_ma_alloc_root, stmt->field_count * sizeof(MYSQL_BIND))))
     {
-      stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+      SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
       return(1);
     }
   }
@@ -1335,7 +1357,7 @@ my_bool STDCALL mysql_stmt_bind_result(MYSQL_STMT *stmt, MYSQL_BIND *bind)
     if (stmt->mysql->methods->db_supported_buffer_type &&
         !stmt->mysql->methods->db_supported_buffer_type(bind[i].buffer_type))
     {
-      stmt_set_error(stmt, CR_UNSUPPORTED_PARAM_TYPE, SQLSTATE_UNKNOWN, 0);
+      SET_CLIENT_STMT_ERROR(stmt, CR_UNSUPPORTED_PARAM_TYPE, SQLSTATE_UNKNOWN, 0);
       return(1);
     }
 
@@ -1476,13 +1498,13 @@ int STDCALL mysql_stmt_fetch(MYSQL_STMT *stmt)
 
   if (stmt->state <= MYSQL_STMT_EXECUTED)
   {
-    stmt_set_error(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
   if (stmt->state < MYSQL_STMT_WAITING_USE_OR_STORE || !stmt->field_count)
   {
-    stmt_set_error(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
     return(1);
   } else if (stmt->state== MYSQL_STMT_WAITING_USE_OR_STORE)
   {
@@ -1512,7 +1534,7 @@ int STDCALL mysql_stmt_fetch_column(MYSQL_STMT *stmt, MYSQL_BIND *bind, unsigned
 {
   if (stmt->state < MYSQL_STMT_USER_FETCHING || column >= stmt->field_count ||
       stmt->state == MYSQL_STMT_FETCH_DONE)  {
-    stmt_set_error(stmt, CR_NO_DATA, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_NO_DATA, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -1618,6 +1640,53 @@ my_bool mthd_stmt_read_prepare_response(MYSQL_STMT *stmt)
   p++;
   /* for backward compatibility we also update mysql->warning_count */
   stmt->mysql->warning_count= stmt->upsert_status.warning_count= uint2korr(p);
+
+/* metadata not supported yet */
+
+  if (stmt->param_count &&
+      stmt->mysql->methods->db_stmt_get_param_metadata(stmt))
+  {
+    return 1;
+  }
+
+  /* allocated bind buffer for parameters */
+  if (stmt->field_count &&
+      stmt->mysql->methods->db_stmt_get_result_metadata(stmt))
+  {
+    return 1;
+  }
+  if (stmt->param_count)
+  {
+    if (stmt->prebind_params)
+    {
+      if (stmt->prebind_params != stmt->param_count)
+      {
+        SET_CLIENT_STMT_ERROR(stmt, CR_INVALID_PARAMETER_NO, SQLSTATE_UNKNOWN, 0);
+        stmt->param_count= stmt->prebind_params;
+        return 1;
+      }
+    } else {
+      if (!(stmt->params= (MYSQL_BIND *)ma_alloc_root(&stmt->mem_root, stmt->param_count * sizeof(MYSQL_BIND))))
+      {
+        SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+        return 1;
+      }
+      memset(stmt->params, '\0', stmt->param_count * sizeof(MYSQL_BIND));
+    }
+  }
+  /* allocated bind buffer for result */
+  if (stmt->field_count)
+  {
+    MA_MEM_ROOT *fields_ma_alloc_root= &((MADB_STMT_EXTENSION *)stmt->extension)->fields_ma_alloc_root;
+    if (!(stmt->bind= (MYSQL_BIND *)ma_alloc_root(fields_ma_alloc_root, stmt->field_count * sizeof(MYSQL_BIND))))
+    {
+      SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+      return 1;
+    }
+    memset(stmt->bind, 0, sizeof(MYSQL_BIND) * stmt->field_count);
+  }
+  stmt->state = MYSQL_STMT_PREPARED;
+
   return(0);
 }
 
@@ -1660,7 +1729,7 @@ int STDCALL mysql_stmt_prepare(MYSQL_STMT *stmt, const char *query, unsigned lon
 
   if (!stmt->mysql)
   {
-    stmt_set_error(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -1692,7 +1761,8 @@ int STDCALL mysql_stmt_prepare(MYSQL_STMT *stmt, const char *query, unsigned lon
 
     stmt->param_count= 0;
     stmt->field_count= 0;
-    stmt->params= 0;
+    stmt->fields= NULL;
+    stmt->params= NULL;
 
     int4store(stmt_id, stmt->stmt_id);
     if (mysql->methods->db_command(mysql, COM_STMT_CLOSE, stmt_id,
@@ -1706,57 +1776,14 @@ int STDCALL mysql_stmt_prepare(MYSQL_STMT *stmt, const char *query, unsigned lon
     if (ma_multi_command(mysql, COM_MULTI_END))
       goto fail;
   
-  if (mysql->net.extension->multi_status > COM_MULTI_OFF)
+  if (mysql->net.extension->multi_status > COM_MULTI_OFF ||
+      mysql->options.extension->skip_read_response)
     return 0;
 
   if (mysql->methods->db_read_prepare_response &&
       mysql->methods->db_read_prepare_response(stmt))
     goto fail;
 
-  /* metadata not supported yet */
-
-  if (stmt->param_count &&
-      stmt->mysql->methods->db_stmt_get_param_metadata(stmt))
-  {
-    goto fail;
-  }
-
-  /* allocated bind buffer for parameters */
-  if (stmt->field_count &&
-      stmt->mysql->methods->db_stmt_get_result_metadata(stmt))
-  {
-    goto fail;
-  }
-  if (stmt->param_count)
-  {
-    if (stmt->prebind_params)
-    {
-      if (stmt->prebind_params != stmt->param_count)
-      {
-        stmt_set_error(stmt, CR_INVALID_PARAMETER_NO, SQLSTATE_UNKNOWN, 0);
-        goto fail;
-      }
-    } else {
-      if (!(stmt->params= (MYSQL_BIND *)ma_alloc_root(&stmt->mem_root, stmt->param_count * sizeof(MYSQL_BIND))))
-      {
-        stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
-        goto fail;
-      }
-      memset(stmt->params, '\0', stmt->param_count * sizeof(MYSQL_BIND));
-    }
-  }
-  /* allocated bind buffer for result */
-  if (stmt->field_count)
-  {
-    MA_MEM_ROOT *fields_ma_alloc_root= &((MADB_STMT_EXTENSION *)stmt->extension)->fields_ma_alloc_root;
-    if (!(stmt->bind= (MYSQL_BIND *)ma_alloc_root(fields_ma_alloc_root, stmt->field_count * sizeof(MYSQL_BIND))))
-    {
-      stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
-      goto fail;
-    }
-    memset(stmt->bind, 0, sizeof(MYSQL_BIND) * stmt->field_count);
-  }
-  stmt->state = MYSQL_STMT_PREPARED;
   return(0);
 
 fail:
@@ -1771,7 +1798,7 @@ int STDCALL mysql_stmt_store_result(MYSQL_STMT *stmt)
 
   if (!stmt->mysql)
   {
-    stmt_set_error(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -1785,7 +1812,7 @@ int STDCALL mysql_stmt_store_result(MYSQL_STMT *stmt)
   if (stmt->state < MYSQL_STMT_EXECUTED)
   {
     SET_CLIENT_ERROR(stmt->mysql, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
-    stmt_set_error(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -1808,7 +1835,7 @@ int STDCALL mysql_stmt_store_result(MYSQL_STMT *stmt)
   else if (stmt->mysql->status != MYSQL_STATUS_STMT_RESULT)
   {
     SET_CLIENT_ERROR(stmt->mysql, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
-    stmt_set_error(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -1848,89 +1875,95 @@ int STDCALL mysql_stmt_store_result(MYSQL_STMT *stmt)
 
 static int madb_alloc_stmt_fields(MYSQL_STMT *stmt)
 {
-  uint i;
   MA_MEM_ROOT *fields_ma_alloc_root= &((MADB_STMT_EXTENSION *)stmt->extension)->fields_ma_alloc_root;
+  MYSQL *mysql= stmt->mysql;
+  if (!mysql->field_count)
+    return 0;
 
-  if (stmt->mysql->field_count)
+  stmt->field_count= mysql->field_count;
+  if (mysql->fields)
   {
+    /* Column info was sent by server */
     ma_free_root(fields_ma_alloc_root, MYF(0));
-    if (!(stmt->fields= (MYSQL_FIELD *)ma_alloc_root(fields_ma_alloc_root,
-            sizeof(MYSQL_FIELD) * stmt->mysql->field_count)))
+    if (!(stmt->fields= ma_duplicate_resultset_metadata(
+              mysql->fields, mysql->field_count,
+              fields_ma_alloc_root)))
     {
-      stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+      SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
       return(1);
     }
-    stmt->field_count= stmt->mysql->field_count;
-
-    for (i=0; i < stmt->field_count; i++)
-    {
-      if (stmt->mysql->fields[i].db)
-        stmt->fields[i].db= ma_strdup_root(fields_ma_alloc_root, stmt->mysql->fields[i].db);
-      if (stmt->mysql->fields[i].table)
-        stmt->fields[i].table= ma_strdup_root(fields_ma_alloc_root, stmt->mysql->fields[i].table);
-      if (stmt->mysql->fields[i].org_table)
-        stmt->fields[i].org_table= ma_strdup_root(fields_ma_alloc_root, stmt->mysql->fields[i].org_table);
-      if (stmt->mysql->fields[i].name)
-        stmt->fields[i].name= ma_strdup_root(fields_ma_alloc_root, stmt->mysql->fields[i].name);
-      if (stmt->mysql->fields[i].org_name)
-        stmt->fields[i].org_name= ma_strdup_root(fields_ma_alloc_root, stmt->mysql->fields[i].org_name);
-      if (stmt->mysql->fields[i].catalog)
-        stmt->fields[i].catalog= ma_strdup_root(fields_ma_alloc_root, stmt->mysql->fields[i].catalog);
-      stmt->fields[i].def= stmt->mysql->fields[i].def ? ma_strdup_root(fields_ma_alloc_root, stmt->mysql->fields[i].def) : NULL;
-      stmt->fields[i].type= stmt->mysql->fields[i].type;
-      stmt->fields[i].length= stmt->mysql->fields[i].length;
-      stmt->fields[i].flags= stmt->mysql->fields[i].flags;
-      stmt->fields[i].decimals= stmt->mysql->fields[i].decimals;
-      stmt->fields[i].charsetnr= stmt->mysql->fields[i].charsetnr;
-      stmt->fields[i].max_length= stmt->mysql->fields[i].max_length;
-      stmt->fields[i].extension=
-                stmt->mysql->fields[i].extension ?
-                ma_field_extension_deep_dup(fields_ma_alloc_root,
-                                            stmt->mysql->fields[i].extension) :
-                NULL;
-    }
-    if (!(stmt->bind= (MYSQL_BIND *)ma_alloc_root(fields_ma_alloc_root, stmt->field_count * sizeof(MYSQL_BIND))))
-    {
-      stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
-      return(1);
-    }
-    memset(stmt->bind, 0, stmt->field_count * sizeof(MYSQL_BIND));
-    stmt->bind_result_done= 0;
+    if (!(stmt->bind= (MYSQL_BIND *) ma_alloc_root(
+        fields_ma_alloc_root, stmt->field_count * sizeof(MYSQL_BIND))))
+     {
+      SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+      return (1);
+     }
   }
+  memset(stmt->bind, 0, stmt->field_count * sizeof(MYSQL_BIND));
+  stmt->bind_result_done= 0;
   return(0);
 }
 
-int stmt_read_execute_response(MYSQL_STMT *stmt)
+int mthd_stmt_read_execute_response(MYSQL_STMT *stmt)
 {
   MYSQL *mysql= stmt->mysql;
   int ret;
+  unsigned int last_status= mysql->server_status;
 
   if (!mysql)
     return(1);
 
-  ret= test((mysql->methods->db_read_stmt_result &&
-                 mysql->methods->db_read_stmt_result(mysql)));
   /* if a reconnect occurred, our connection handle is invalid */
   if (!stmt->mysql)
-    return(1);
+    return (1);
+
+  ret= test((mysql->methods->db_read_stmt_result &&
+                 mysql->methods->db_read_stmt_result(mysql)));
+  
+  if (!ret && mysql->field_count && !mysql->fields)
+  {
+      /*
+        Column info was not sent by server, copy
+        from stmt->fields
+      */
+      assert(stmt->fields);
+      /*
+         Too bad, C/C resets stmt->field_count to 0
+         before reading SP output variables result sets.
+      */
+      if(!stmt->field_count)
+        stmt->field_count = mysql->field_count;
+      else
+        assert(mysql->field_count == stmt->field_count);
+      mysql->fields= ma_duplicate_resultset_metadata(
+          stmt->fields, stmt->field_count, &mysql->field_alloc);
+      if (!mysql->fields)
+      {
+        SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+        return (1);
+      }
+  }
 
   /* update affected rows, also if an error occurred */
   stmt->upsert_status.affected_rows= stmt->mysql->affected_rows;
 
   if (ret)
   {
-    stmt_set_error(stmt, mysql->net.last_errno, mysql->net.sqlstate,
+    SET_CLIENT_STMT_ERROR(stmt, mysql->net.last_errno, mysql->net.sqlstate,
        mysql->net.last_error);
     /* if mariadb_stmt_execute_direct was used, we need to send the number
        of parameters to the specified prebinded value to prevent possible
        memory overrun */
     if (stmt->prebind_params)
+    {
       stmt->param_count= stmt->prebind_params;
+    }
     stmt->state= MYSQL_STMT_PREPARED;
     return(1);
   }
   stmt->upsert_status.last_insert_id= mysql->insert_id;
   stmt->upsert_status.server_status= mysql->server_status;
+  ma_status_callback(stmt->mysql, last_status);
   stmt->upsert_status.warning_count= mysql->warning_count;
 
   CLEAR_CLIENT_ERROR(mysql);
@@ -1956,7 +1989,7 @@ int stmt_read_execute_response(MYSQL_STMT *stmt)
           !(stmt->fields= (MYSQL_FIELD *)ma_alloc_root(fields_ma_alloc_root,
               sizeof(MYSQL_FIELD) * mysql->field_count)))
       {
-        stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+        SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
         return(1);
       }
       memset(stmt->bind, 0, sizeof(MYSQL_BIND) * mysql->field_count);
@@ -2041,7 +2074,7 @@ int stmt_read_execute_response(MYSQL_STMT *stmt)
     } else
     {
       /* table was altered, see test_wl4166_2  */
-      stmt_set_error(stmt, CR_NEW_STMT_METADATA, SQLSTATE_UNKNOWN, 0);
+      SET_CLIENT_STMT_ERROR(stmt, CR_NEW_STMT_METADATA, SQLSTATE_UNKNOWN, 0);
       return(1);
     }
   }
@@ -2057,20 +2090,20 @@ int STDCALL mysql_stmt_execute(MYSQL_STMT *stmt)
 
   if (!stmt->mysql)
   {
-    stmt_set_error(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
   if (stmt->state < MYSQL_STMT_PREPARED)
   {
     SET_CLIENT_ERROR(mysql, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
-    stmt_set_error(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
   if (stmt->param_count && !stmt->bind_param_done)
   {
-    stmt_set_error(stmt, CR_PARAMS_NOT_BOUND, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_PARAMS_NOT_BOUND, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -2097,11 +2130,8 @@ int STDCALL mysql_stmt_execute(MYSQL_STMT *stmt)
   }
   /* CONC-344: set row count to zero */
   stmt->result.rows= 0;
-  if (stmt->array_size > 0)
-    request= (char *)mysql_stmt_execute_generate_bulk_request(stmt, &request_len);
-  else
-    request= (char *)mysql_stmt_execute_generate_simple_request(stmt, &request_len);
 
+  request= (char *)ma_stmt_execute_generate_request(stmt, &request_len, 0);
   if (!request)
     return 1;
 
@@ -2117,10 +2147,11 @@ int STDCALL mysql_stmt_execute(MYSQL_STMT *stmt)
     return(1);
   }
 
-  if (mysql->net.extension->multi_status > COM_MULTI_OFF)
+  if (mysql->net.extension->multi_status > COM_MULTI_OFF ||
+      mysql->options.extension->skip_read_response)
     return(0);
 
-  return(stmt_read_execute_response(stmt));
+  return(mthd_stmt_read_execute_response(stmt));
 }
 
 static my_bool madb_reset_stmt(MYSQL_STMT *stmt, unsigned int flags)
@@ -2130,7 +2161,7 @@ static my_bool madb_reset_stmt(MYSQL_STMT *stmt, unsigned int flags)
 
   if (!stmt->mysql)
   {
-    stmt_set_error(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -2209,13 +2240,14 @@ static my_bool mysql_stmt_internal_reset(MYSQL_STMT *stmt, my_bool is_close)
 {
   MYSQL *mysql= stmt->mysql;
   my_bool ret= 1;
-  unsigned int flags= MADB_RESET_LONGDATA | MADB_RESET_BUFFER | MADB_RESET_ERROR;
+  unsigned int flags= MADB_RESET_LONGDATA | MADB_RESET_BUFFER | MADB_RESET_ERROR,
+               last_status= mysql->server_status;
 
   if (!mysql)
   {
     /* connection could be invalid, e.g. after mysql_stmt_close or failed reconnect
        attempt (see bug CONC-97) */
-    stmt_set_error(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -2254,6 +2286,7 @@ static my_bool mysql_stmt_internal_reset(MYSQL_STMT *stmt, my_bool is_close)
   stmt->upsert_status.affected_rows= mysql->affected_rows;
   stmt->upsert_status.last_insert_id= mysql->insert_id;
   stmt->upsert_status.server_status= mysql->server_status;
+  ma_status_callback(stmt->mysql, last_status);
   stmt->upsert_status.warning_count= mysql->warning_count;
   mysql->status= MYSQL_STATUS_READY;
 
@@ -2267,10 +2300,10 @@ MYSQL_RES * STDCALL mysql_stmt_result_metadata(MYSQL_STMT *stmt)
   if (!stmt->field_count)
     return(NULL);
 
-  /* aloocate result set structutr and copy stmt information */
+  /* allocate result set structure and copy stmt information */
   if (!(res= (MYSQL_RES *)calloc(1, sizeof(MYSQL_RES))))
   {
-    stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
     return(NULL);
   }
 
@@ -2321,13 +2354,13 @@ my_bool STDCALL mysql_stmt_send_long_data(MYSQL_STMT *stmt, uint param_number,
 
   if (stmt->state < MYSQL_STMT_PREPARED || !stmt->params)
   {
-    stmt_set_error(stmt, CR_NO_PREPARE_STMT, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_NO_PREPARE_STMT, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
   if (param_number >= stmt->param_count)
   {
-    stmt_set_error(stmt, CR_INVALID_PARAMETER_NO, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_INVALID_PARAMETER_NO, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -2387,14 +2420,14 @@ int STDCALL mysql_stmt_next_result(MYSQL_STMT *stmt)
 
   if (!stmt->mysql)
   {
-    stmt_set_error(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
   if (stmt->state < MYSQL_STMT_EXECUTED)
   {
     SET_CLIENT_ERROR(stmt->mysql, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
-    stmt_set_error(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
     return(1);
   }
 
@@ -2409,7 +2442,7 @@ int STDCALL mysql_stmt_next_result(MYSQL_STMT *stmt)
   if (mysql_next_result(stmt->mysql))
   {
     stmt->state= MYSQL_STMT_FETCH_DONE;
-    stmt_set_error(stmt, stmt->mysql->net.last_errno, stmt->mysql->net.sqlstate,
+    SET_CLIENT_STMT_ERROR(stmt, stmt->mysql->net.last_errno, stmt->mysql->net.sqlstate,
         stmt->mysql->net.last_error);
     return(1);
   }
@@ -2421,9 +2454,11 @@ int STDCALL mysql_stmt_next_result(MYSQL_STMT *stmt)
     rc= madb_alloc_stmt_fields(stmt);
   else
   {
+    unsigned int last_status= stmt->mysql->server_status;
     stmt->upsert_status.affected_rows= stmt->mysql->affected_rows;
     stmt->upsert_status.last_insert_id= stmt->mysql->insert_id;
     stmt->upsert_status.server_status= stmt->mysql->server_status;
+    ma_status_callback(stmt->mysql, last_status);
     stmt->upsert_status.warning_count= stmt->mysql->warning_count;
   }
 
@@ -2447,7 +2482,7 @@ int STDCALL mariadb_stmt_execute_direct(MYSQL_STMT *stmt,
   mysql= stmt->mysql;
   if (!mysql)
   {
-    stmt_set_error(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_SERVER_LOST, SQLSTATE_UNKNOWN, 0);
     return 1;
   }
 
@@ -2469,7 +2504,7 @@ int STDCALL mariadb_stmt_execute_direct(MYSQL_STMT *stmt,
 
   if (ma_multi_command(mysql, COM_MULTI_ENABLED))
   {
-    stmt_set_error(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
+    SET_CLIENT_STMT_ERROR(stmt, CR_COMMANDS_OUT_OF_SYNC, SQLSTATE_UNKNOWN, 0);
     return 1;
   }
 
@@ -2522,43 +2557,18 @@ int STDCALL mariadb_stmt_execute_direct(MYSQL_STMT *stmt,
   if (ma_multi_command(mysql, COM_MULTI_END))
     goto fail;
 
-  /* read prepare response */
-  if (mysql->methods->db_read_prepare_response &&
-    mysql->methods->db_read_prepare_response(stmt))
-  goto fail;
-
-  clear_result= 0;
-
-  /* metadata not supported yet */
-
-  if (stmt->param_count &&
-      stmt->mysql->methods->db_stmt_get_param_metadata(stmt))
+  if (!mysql->options.extension->skip_read_response)
   {
+    /* read prepare response */
+    if (mysql->methods->db_read_prepare_response &&
+      mysql->methods->db_read_prepare_response(stmt))
     goto fail;
-  }
 
-  /* allocated bind buffer for parameters */
-  if (stmt->field_count &&
-      stmt->mysql->methods->db_stmt_get_result_metadata(stmt))
-  {
-    goto fail;
-  }
+    clear_result= 0;
 
-  /* allocated bind buffer for result */
-  if (stmt->field_count)
-  {
-    MA_MEM_ROOT *fields_ma_alloc_root= &((MADB_STMT_EXTENSION *)stmt->extension)->fields_ma_alloc_root;
-    if (!(stmt->bind= (MYSQL_BIND *)ma_alloc_root(fields_ma_alloc_root, stmt->field_count * sizeof(MYSQL_BIND))))
-    {
-      stmt_set_error(stmt, CR_OUT_OF_MEMORY, SQLSTATE_UNKNOWN, 0);
-      goto fail;
-    }
-    memset(stmt->bind, 0, sizeof(MYSQL_BIND) * stmt->field_count);
+    /* read execute response packet */
+    return mthd_stmt_read_execute_response(stmt);
   }
-  stmt->state = MYSQL_STMT_PREPARED;
-
-  /* read execute response packet */
-  return stmt_read_execute_response(stmt);
 fail:
   /* check if we need to set error message */
   if (!mysql_stmt_errno(stmt))
@@ -2568,14 +2578,7 @@ fail:
       stmt->mysql->methods->db_stmt_flush_unbuffered(stmt);
     } while(mysql_stmt_more_results(stmt));
   }
-
-  /* CONC-633: If prepare returned an error, we ignore error from execute */
-  if (mysql_stmt_errno(stmt))
-  {
-    my_set_error(mysql, mysql_stmt_errno(stmt), mysql_stmt_sqlstate(stmt),
-                 mysql_stmt_error(stmt));
-    stmt->state= MYSQL_STMT_INITTED;
-  } 
+  stmt->state= MYSQL_STMT_INITTED;
   return 1;
 }
 
